package net.codinux.banking.client

import net.codinux.banking.client.model.AccountCredentials
import net.codinux.banking.client.model.Amount
import net.codinux.banking.client.model.BankAccount
import net.codinux.banking.client.model.BankAccess
import net.codinux.banking.client.model.options.GetAccountDataOptions
import net.codinux.banking.client.model.request.GetAccountDataRequest
import net.codinux.banking.client.model.request.TransferMoneyRequest
import net.codinux.banking.client.model.request.TransferMoneyRequestForUser
import net.codinux.banking.client.model.response.GetTransactionsResponse
import net.codinux.banking.client.model.response.Response
import net.codinux.banking.client.model.tan.TanMethodType

abstract class BankingClientForUserBase(
    protected val credentials: AccountCredentials,
    protected val client: BankingClient
) : BankingClientForUser {

    private lateinit var bank: BankAccess

    override suspend fun getAccountDataAsync(options: GetAccountDataOptions) =
        client.getAccountDataAsync(GetAccountDataRequest(credentials, options)).also {
            it.data?.bank?.let { retrievedBank ->
                this.bank = retrievedBank
            }
        }

    override suspend fun updateAccountTransactionsAsync(accounts: List<BankAccount>?, preferredTanMethodsIfSelectedTanMethodIsNotAvailable: List<TanMethodType>?): Response<List<GetTransactionsResponse>> =
        client.updateAccountTransactionsAsync(bank, accounts)


    override suspend fun transferMoneyAsync(recipientName: String, recipientAccountIdentifier: String, amount: Amount, paymentReference: String?) =
        transferMoneyAsync(TransferMoneyRequest(null, recipientName, recipientAccountIdentifier, null, amount, paymentReference = paymentReference))

    override suspend fun transferMoneyAsync(request: TransferMoneyRequest) =
        client.transferMoneyAsync(TransferMoneyRequestForUser(bank.domesticBankCode, bank.loginName, bank.password!!, request))

}