package net.codinux.banking.client

import kotlinx.coroutines.runBlocking
import net.codinux.banking.client.model.Amount
import net.codinux.banking.client.model.BankAccount
import net.codinux.banking.client.model.BankAccess
import net.codinux.banking.client.model.options.GetAccountDataOptions
import net.codinux.banking.client.model.request.GetAccountDataRequest
import net.codinux.banking.client.model.request.TransferMoneyRequest
import net.codinux.banking.client.model.request.TransferMoneyRequestForUser
import net.codinux.banking.client.model.tan.TanMethodType

/*      BankingClient       */

fun BankingClient.getAccountData(bankCode: String, loginName: String, password: String) = runBlocking {
    getAccountDataAsync(bankCode, loginName, password)
}

fun BankingClient.getAccountData(request: GetAccountDataRequest) = runBlocking {
    getAccountDataAsync(request)
}

fun BankingClient.updateAccountTransactions(bank: BankAccess, accounts: List<BankAccount>? = null, preferredTanMethodsIfSelectedTanMethodIsNotAvailable: List<TanMethodType>? = TanMethodType.TanMethodsPreferredByMostApplications) = runBlocking {
    updateAccountTransactionsAsync(bank, accounts, preferredTanMethodsIfSelectedTanMethodIsNotAvailable)
}


fun BankingClient.transferMoney(bankCode: String, loginName: String, password: String, recipientName: String,
                               recipientAccountIdentifier: String, amount: Amount, paymentReference: String? = null) = runBlocking {
    transferMoneyAsync(bankCode, loginName, password, recipientName, recipientAccountIdentifier, amount, paymentReference)
}

fun BankingClient.transferMoney(request: TransferMoneyRequestForUser) = runBlocking {
    transferMoneyAsync(request)
}


/*      BankingClientForUser        */

fun BankingClientForUser.getAccountData() = runBlocking {
    getAccountDataAsync()
}

fun BankingClientForUser.getAccountData(options: GetAccountDataOptions) = runBlocking {
    getAccountDataAsync(options)
}

fun BankingClientForUser.updateAccountTransactions(accounts: List<BankAccount>? = null, preferredTanMethodsIfSelectedTanMethodIsNotAvailable: List<TanMethodType>? = TanMethodType.TanMethodsPreferredByMostApplications) = runBlocking {
    updateAccountTransactionsAsync(accounts, preferredTanMethodsIfSelectedTanMethodIsNotAvailable)
}


fun BankingClientForUser.transferMoney(recipientName: String, recipientAccountIdentifier: String, amount: Amount, paymentReference: String? = null) = runBlocking {
    transferMoneyAsync(recipientName, recipientAccountIdentifier, amount, paymentReference)
}

fun BankingClientForUser.transferMoney(request: TransferMoneyRequest) = runBlocking {
    transferMoneyAsync(request)
}