package net.codinux.banking.client.model

import net.codinux.banking.client.model.config.NoArgConstructor

/**
 * Contains only the basic info of a bank, just enough that a client application can display it to the user
 * and the user knows exactly which bank is meant / referred to.
 *
 * As e.g. when adding a new account, client application has no data about the bank locally, so it can use this
 * information to display it to the user.
 */
@NoArgConstructor
open class BankViewInfo(
    val bankCode: String,
    var loginName: String,
    val bankName: String,
    val bankingGroup: BankingGroup? = null
) {
    override fun toString() = "$bankCode $bankName $loginName"
}