package net.codinux.banking.client.model.request

import net.codinux.banking.client.model.*
import net.codinux.banking.client.model.config.NoArgConstructor
import net.codinux.banking.client.model.tan.TanMethodType

/**
 * For documentation see [TransferMoneyRequest].
 */
@NoArgConstructor
open class TransferMoneyRequestForUser(

    /*      Sender settings     */

    val bankCode: String,
    val loginName: String,
    val password: String,

    senderAccount: BankAccountIdentifier? = null,


    /*      Recipient settings     */

    recipientName: String,
    recipientAccountIdentifier: String,
    recipientBankIdentifier: String? = null,


    /*      Transfer data    */

    amount: Amount,
    currency: String = DefaultValues.DefaultCurrency,
    paymentReference: String? = null,

    instantTransfer: Boolean = false,


    preferredTanMethods: List<TanMethodType>? = TanMethodType.TanMethodsPreferredByMostApplications,
    tanMethodsNotSupportedByApplication: List<TanMethodType> = TanMethodType.TanMethodsNotSupportedByMostApplications,

    clientData: Any? = null,
    serializedClientData: String? = null
) : TransferMoneyRequest(senderAccount, recipientName, recipientAccountIdentifier, recipientBankIdentifier, amount, currency, paymentReference, instantTransfer, preferredTanMethods, tanMethodsNotSupportedByApplication, clientData, serializedClientData) {

    constructor(bankCode: String, loginName: String, password: String, request: TransferMoneyRequest)
            : this(bankCode, loginName, password, request.senderAccount, request.recipientName, request.recipientAccountIdentifier, request.recipientBankIdentifier,
                request.amount, request.currency, request.paymentReference, request.instantTransfer, request.preferredTanMethods, request.tanMethodsNotSupportedByApplication)

    constructor(
        bank: BankAccess, account: BankAccount?,
        recipientName: String, recipientAccountIdentifier: String, recipientBankIdentifier: String? = null,
        amount: Amount, currency: String = DefaultValues.DefaultCurrency, paymentReference: String? = null, instantTransfer: Boolean = false,
        preferredTanMethods: List<TanMethodType>? = TanMethodType.TanMethodsPreferredByMostApplications
    ) : this(bank.domesticBankCode, bank.loginName, bank.password!!, account?.let { BankAccountIdentifier(it.identifier, it.subAccountNumber, it.iban) },
        recipientName, recipientAccountIdentifier, recipientBankIdentifier, amount, currency, paymentReference, instantTransfer,
        listOf(bank.selectedTanMethod.type) + (preferredTanMethods ?: emptyList()), TanMethodType.TanMethodsNotSupportedByMostApplications,
        bank.clientData, bank.serializedClientData
    ) {
        this.bank = bank
        this.account = account
    }

    open var bank: BankAccess? = null
        protected set

    open var account: BankAccount? = null
        protected set

    override fun toString() = "$bankCode $loginName ${super.toString()}"
}