package net.codinux.banking.client.model.response

import net.codinux.banking.client.model.MessageLogEntry
import net.codinux.banking.client.model.config.NoArgConstructor

// TODO: may differentiate between ClientResponse, which is either Success or Error, and RestResponse, which can be Success, Error and TanRequired
@NoArgConstructor
open class Response<T> protected constructor(
    val type: ResponseType,
    val data: T? = null,
    val error: Error? = null,
    val tanRequired: TanRequired? = null,
    val messageLog: List<MessageLogEntry> = emptyList()
) {

    companion object {
        fun <T> success(data: T, messageLog: List<MessageLogEntry> = emptyList()): Response<T> =
            Response(ResponseType.Success, data, messageLog = messageLog)

        fun <T> error(errorType: ErrorType, internalError: String? = null, errorMessagesFromBank: List<String> = emptyList(), messageLog: List<MessageLogEntry> = emptyList()): Response<T> =
            Response(ResponseType.Error, null, Error(errorType, internalError, errorMessagesFromBank), messageLog = messageLog)

        fun <T> tanRequired(tanRequired: TanRequired, messageLog: List<MessageLogEntry> = emptyList()): Response<T> =
            Response(ResponseType.TanRequired, null, null, tanRequired, messageLog)

        fun <T> bankReturnedError(errorMessagesFromBank: List<String>, messageLog: List<MessageLogEntry> = emptyList()): Response<T> =
            Response.error(ErrorType.BankReturnedError, null, errorMessagesFromBank, messageLog)
    }


    override fun toString() = when (type) {
        ResponseType.Success -> "Success: $data"
        ResponseType.Error -> "Error: $error"
        ResponseType.TanRequired -> "TanRequired: $tanRequired"
    }
}