package net.codinux.banking.client.model.securitiesaccount

import kotlinx.datetime.Instant
import kotlinx.datetime.LocalDate
import net.codinux.banking.client.model.Amount
import net.codinux.banking.client.model.config.NoArgConstructor

@NoArgConstructor
open class Holding(
    open var name: String,

    open var isin: String? = null,
    open var wkn: String? = null,

    open var quantity: Double? = null,
    open var currency: String? = null,

    /**
     * Gesamter Kurswert aller Einheiten des Wertpapiers
     */
    open var totalBalance: Amount? = null,
    /**
     * Aktueller Kurswert einer einzelnen Einheit des Wertpapiers
     */
    open var marketValue: Amount? = null,

    /**
     * Änderung in Prozent Aktueller Kurswert gegenüber Einstandspreis.
     */
    open var performancePercentage: Float? = null,

    /**
     * Gesamter Einstandspreis (Kaufpreis)
     */
    open var totalCostPrice: Amount? = null,
    /**
     * (Durchschnittlicher) Einstandspreis/-kurs einer Einheit des Wertpapiers
     */
    open var averageCostPrice: Amount? = null,

    /**
     * Zeitpunkt zu dem der Kurswert bestimmt wurde
     */
    open var pricingTime: Instant? = null,

    open var buyingDate: LocalDate? = null,

    var userSetDisplayName: String? = null,
) {
    open val identifier: String by lazy { "${isin}_$wkn" }

    override fun toString() = "$name $totalBalance $currency"
}