package net.codinux.banking.client.model.tan

import net.codinux.banking.client.model.config.NoArgConstructor

@NoArgConstructor
open class TanMedium(
    val type: TanMediumType,
    val mediumName: String?,
    val status: TanMediumStatus,
    /**
     * Only set if [type] is [TanMediumType.TanGenerator].
     */
    val tanGenerator: TanGeneratorTanMedium? = null,
    /**
     * Only set if [type] is [TanMediumType.MobilePhone].
     */
    val mobilePhone: MobilePhoneTanMedium? = null,

    var userSetDisplayName: String? = null
) {

    /**
     * Using only [mediumName] as identifier does not work as e.g. Sparkasse names all their TanGenerator TAN media
     * "SparkassenCard (Debitkarte)" - so it's not possible to differentiate between them solely by medium name.
     */
    val identifier: String by lazy {
        // TODO: translate
        var id = mediumName ?: when (type) {
            TanMediumType.MobilePhone -> "Mobiltelefon"
            TanMediumType.TanGenerator -> "Tan Generator"
            TanMediumType.Generic -> "Unbenanntes TAN Medium"
        }

        if (mobilePhone != null) {
            id += " " + (mobilePhone.concealedPhoneNumber ?: mobilePhone.phoneNumber)
        }
        if (tanGenerator != null) {
            if (tanGenerator.cardNumber.isNotBlank()) {
                id += " " + tanGenerator.cardNumber
            }
            if (tanGenerator.cardSequenceNumber.isNullOrBlank() == false) {
                id += " " + tanGenerator.cardSequenceNumber
            }

            if (tanGenerator.validFrom != null && tanGenerator.validTo != null) {
                id += ", gültig von " + tanGenerator.validFrom.let { "${it.dayOfMonth}.${it.monthNumber}${it.year}" } +
                        " - " + tanGenerator.validTo.let { "${it.dayOfMonth}.${it.monthNumber}${it.year}" }
            } else if (tanGenerator.validTo != null) {
                id += ", gültig bis " + tanGenerator.validTo.let { "${it.dayOfMonth}.${it.monthNumber}${it.year}" }
            }
        }

        id
    }

    val displayName: String by lazy {
        userSetDisplayName
            ?: (identifier + " " + when (status) {
                TanMediumStatus.Used -> "Aktiv"
                TanMediumStatus.Available -> "Verfügbar"
                TanMediumStatus.ActiveFollowUpCard -> " Folgekarte, aktiv bei erster Nutzung"
                TanMediumStatus.AvailableFollowUpCard -> " Folgekarte, die erst aktiviert werden muss"
            })
    }

    override fun toString() = "$mediumName $status"
}