package net.codinux.banking.client.model.tan

enum class TanMethodType {
    EnterTan,

    ChipTanManual,

    ChipTanFlickerCode,

    ChipTanUsb,

    ChipTanQrCode,

    ChipTanPhotoTanMatrixCode,

    SmsTan,

    AppTan,

    DecoupledTan,

    DecoupledPushTan,

    photoTan,

    QrCode
    ;


    val isDecoupledMethod: Boolean
        get() = this == DecoupledTan || this == DecoupledPushTan


    companion object {

        val NonVisual = listOf(TanMethodType.DecoupledTan, TanMethodType.DecoupledPushTan, TanMethodType.AppTan, TanMethodType.SmsTan, TanMethodType.ChipTanManual, TanMethodType.EnterTan)

        val NonVisualWithoutChipTanManual = NonVisual.toMutableList().apply { remove(TanMethodType.ChipTanManual) }.toList()

        val ImageBased = listOf(
            TanMethodType.QrCode, TanMethodType.photoTan, // non ChipTan
            TanMethodType.ChipTanQrCode, TanMethodType.ChipTanPhotoTanMatrixCode // ChipTan; QrCode (e.g. used by Sparkassen) is faster than MatrixCode (e.g. used by Volksbanken)
        )

        val NonVisualOrImageBased = buildList {
            addAll(NonVisualWithoutChipTanManual)
            addAll(ImageBased)
            addAll(listOf(TanMethodType.ChipTanManual)) // this is quite inconvenient for user, so i added it as last
        }

        /**
         * The same as [NonVisualOrImageBased] but including [ChipTanFlickerCode] - for applications supporting it - as
         * FlickerCode is still the most used ChipTan procedure.
         */
        val NonVisualOrImageBasedOrFlickerCode = NonVisualOrImageBased.toMutableList().apply {
            val index = this.indexOf(ChipTanQrCode)
            this.add(index, ChipTanFlickerCode)
        }.toList()


        val TanMethodsPreferredByMostApplications = NonVisualOrImageBased


        val TanMethodsNotSupportedByMostApplications = listOf(TanMethodType.ChipTanUsb)

    }

}