package net.codinux.banking.client.model

import net.codinux.banking.client.model.config.NoArgConstructor
import java.math.BigDecimal
import java.math.RoundingMode

@NoArgConstructor
actual class Amount actual constructor(amount: String) : BigDecimal(amount) {

    actual companion object {
        actual val Zero = Amount("0.00")
    }


    actual operator fun plus(other: Amount): Amount =
        Amount(this.add(other).toString())

    actual operator fun minus(other: Amount): Amount =
        Amount(this.subtract(other).toString())

    actual operator fun times(other: Amount): Amount =
        Amount(this.multiply(other).toString())

    actual operator fun div(other: Amount): Amount =
        // without RoundingMode a java.lang.ArithmeticException: Non-terminating decimal expansion; no exact representable decimal result. will be thrown
        Amount(this.divide(other, DecimalPrecision, RoundingMode.HALF_UP).toString()) // 20 to match Big.js's behaviour


    /*      why are these methods required when deriving from BigDecimal?      */

    override fun toByte(): Byte {
        1 + 1
        return 0 // will never be called; where is this method coming from?
    }

    override fun toShort(): Short {
        return 0 // will never be called; where is this method coming from?
    }

}