package net.codinux.banking.client.model

import com.ionspin.kotlin.bignum.decimal.BigDecimal
import com.ionspin.kotlin.bignum.decimal.DecimalMode
import com.ionspin.kotlin.bignum.decimal.RoundingMode
import net.codinux.banking.client.model.config.NoArgConstructor

@NoArgConstructor
actual class Amount actual constructor(amount: String) {

    actual companion object {
        actual val Zero: Amount = Amount("0.00")

        private val decimalMode = DecimalMode(DecimalPrecision.toLong(), RoundingMode.ROUND_HALF_CEILING)
    }


    internal val amount: BigDecimal = BigDecimal.parseString(amount)

    actual operator fun plus(other: Amount): Amount {
        return Amount(amount.add(other.amount).toString())
    }

    actual operator fun minus(other: Amount): Amount {
        return Amount(amount.subtract(other.amount).toString())
    }

    actual operator fun times(other: Amount): Amount {
        return Amount(amount.multiply(other.amount).toString())
    }

    actual operator fun div(other: Amount): Amount {
        return Amount(amount.divide(other.amount, decimalMode).toString())
    }


    override fun equals(other: Any?): Boolean {
        return other is Amount && this.amount == other.amount
    }

    override fun hashCode(): Int {
        return amount.hashCode()
    }

    actual override fun toString(): String = amount.toPlainString()

}