package net.codinux.banking.client.fints4k

import net.codinux.banking.client.BankingClientCallback
import net.codinux.banking.client.model.MessageLogEntryType
import net.codinux.banking.fints.callback.FinTsClientCallback
import net.codinux.banking.fints.messages.datenelemente.implementierte.tan.TanMedium
import net.codinux.banking.fints.model.BankData
import net.codinux.banking.fints.model.EnterTanGeneratorAtcResult
import net.codinux.banking.fints.model.MessageLogEntry
import net.codinux.banking.fints.model.TanMethod

open class BridgeFintTsToBankingClientCallback(
    protected val bankingClientCallback: BankingClientCallback,
    protected val mapper: FinTs4kMapper
) : FinTsClientCallback {

    override suspend fun askUserForTanMethod(supportedTanMethods: List<TanMethod>, suggestedTanMethod: TanMethod?): TanMethod? {
        return suggestedTanMethod
    }

    override suspend fun enterTan(tanChallenge: net.codinux.banking.fints.model.TanChallenge) {
        bankingClientCallback.enterTan(mapper.mapTanChallenge(tanChallenge)) { enterTanResult ->
            if (enterTanResult.enteredTan != null) {
                tanChallenge.userEnteredTan(enterTanResult.enteredTan!!)
            } else if (enterTanResult.changeTanMethodTo != null) {
                val fintsTanMethod = tanChallenge.bank.tanMethodsAvailableForUser.first { it.securityFunction.code == enterTanResult.changeTanMethodTo!!.identifier }
                tanChallenge.userAsksToChangeTanMethod(fintsTanMethod)
            } else {
                tanChallenge.userDidNotEnterTan()
            }
        }
    }

    override suspend fun enterTanGeneratorAtc(bank: BankData, tanMedium: TanMedium): EnterTanGeneratorAtcResult {
        return EnterTanGeneratorAtcResult.userDidNotEnterAtc()
    }

    override fun messageLogAdded(messageLogEntry: MessageLogEntry) {
        val mapped = mapper.mapMessageLogEntry(messageLogEntry)

        bankingClientCallback.messageLogAdded(mapped)
    }
}