package net.codinux.banking.client.fints4k

import net.codinux.banking.client.model.Amount
import kotlin.test.Test
import net.codinux.banking.fints.transactions.swift.model.Holding
import kotlin.test.assertEquals

class FinTs4kMapperTest {

    private val underTest = FinTs4kMapper()


    @Test
    fun getTotalBalance_TotalBalanceIsNull_CalculateByQuantityAndMarketValue() {
        val holding = Holding("", null, null, null, 4.0, null, null, null, fints4kAmount("13.33"))

        val result = underTest.getTotalBalance(holding)

        assertEquals(Amount("53.32"), result)
    }

    @Test
    fun getTotalCostPrice_TotalCostPriceIsNull_CalculateByQuantityAndAverageCostPrice() {
        val holding = Holding("", null, null, null, 47.0, fints4kAmount("16.828"), null)

        val result = underTest.getTotalCostPrice(holding)

        assertEquals(Amount("790.9159999999999"), result)
    }

    @Test
    fun calculatePerformance_ByTotalBalanceAndTotalCostPrice() {
        val holding = Holding("", null, null, null, null, null, fints4kAmount("20217.12"), null, null, null, fints4kAmount("19027.04"))

        val result = underTest.calculatePerformance(holding)

        assertEquals(6.2546773f, result!!, 0.000001f) // for JS the result has too many decimal places, so add a tolerance
    }

    @Test
    fun calculatePerformance_ByMarketValueAndAverageCostPrice() {
        val holding = Holding("", null, null, null, null, fints4kAmount("16.828"), null, null, fints4kAmount("16.75"), null, null)

        val result = underTest.calculatePerformance(holding)

        assertEquals(-0.4635132f, result!!, 0.0000001f) // for JS the result has too many decimal places, so add a tolerance
    }


    private fun fints4kAmount(amount: String) =
        net.codinux.banking.fints.model.Amount(amount)

}